from PyNovCom import *

class PyEps:

    def __init__(self, ip_addr=""):
        self.isConnected = False
        self.dev = PyNovCom(ip_addr)


    # basic methods:

    def open(self):
        self.dev.open()
        self.isConnected = self.dev.isConnected

    def read(self, addr):
        if self.dev.isConnected:
            data = self.dev.read(addr)
        else:
            data = 0
        return data

    def write(self, addr, data):
        if self.dev.isConnected:
            self.dev.write(addr, data)

    def close(self):
        self.isConnected = False
        if self.dev.isConnected:
            self.dev.close()


    # instrument-specific methods:

    def getfirmware(self): # as string
        return hex(self.read(84))

    def getserialnumber(self): # as integer
        return self.read(91)

    def getmoduletype(self): # as string
         str="";
         for ii in range(16):
            dummy = self.read(96+ii)
            str += chr(dummy >> 8)
            str += chr(dummy & 0xFF)

         return str

    def gettemperature(self): # as double
        return self.read(181)/16

    def getnrofwlenbands(self): # as integer
        return self.read(190)

    def getoptfrequency(self): # in THz
        minfreq = self.read(193)/2**7
        maxfreq = self.read(192)/2**7
        steps = self.read(194)
        stepsize = (maxfreq-minfreq)/steps
        index = self.read(25)
        freq = minfreq+stepsize*index
        return freq

    def setoptfrequency(self, freq): # in THz
         minfreq = self.read(193)/2**7
         maxfreq = self.read(192)/2**7
         steps = self.read(194)
         stepsize = (maxfreq-minfreq)/steps
         index = round((freq-minfreq)/stepsize)
         if (index>=0) & (index<=steps):
            self.write(25, index)
         else:
            print("'Freq. exceeds range!")

    def setposition(self, wp, position): # as integer 0..2^16-1
         #0: HWP
         #1..6: QWP0..QWP5
         self.write(40+wp, round(position))

    def getposition(self, wp): # as integer 0..2^16-1
         #0: HWP
         #1..6: QWP0..QWP5
         return self.read(40+wp)


    def setspeed(self, wp, speed): # nominal speed in rad/s
         #0: HWP
         #1..6: QWP0..QWP5
         self.write(10+2*wp, 0) # clear MSB register first to avoid overflow
         
         if wp==0: # HWP
            speed = min(20000000, speed) # 20 Mrad/s max
            speed_fpga = round(speed/10)
         else:
            speed = min(2000000, speed) # 2 Mrad/s max
            speed_fpga = round(speed*100)
         
         #print(speed_fpga)
         self.write(9+2*wp, speed_fpga & 0xFFFF) # LSB
         self.write(10+2*wp, speed_fpga >> 16) # MSB

    def getspeed(self, wp): # nominal speed in rad/s
         #0: HWP
         #1..6: QWP0..QWP5
         r = self.read(10+2*wp)*2**16 + self.read(9+2*wp)
         if wp==0: # HWP
            r = r*10
         else:
            r = r/100

         return r
    
    def setrotation(self, wp, backw_en):
        #0: HWP
        #1..6: QWP0..QWP5
        #0, 2: STOP
        #1, FORWARD
        #3, BACKWARD
        self.write(wp, backw_en)
      
    def setrotation_all(self, backw_en):
        #backw_en is an array of 7 values for 7 waveplates
        val = 0
        for x in backw_en:
            val = (val<<2) + x
            #print(x)
        
        self.write(7, val)
        #print(hex(val))
      
         
      

